/*
 * clocks_and_modes.c              Copyright NXP 2016
 * Description: Example clock and mode functions
 * 2015 Feb 22 S Mihalik/ O Romero - initial version
 * 2016 Oct 15 S.M. / O.R. - Updated PLL per S32K1XX data sheet rev 1
 * 2016 Dec 06 S.M. et al - NormalRUNmode_80MHz changed "if" to "while"
 * 2016 Jan 05 S.M. - NormalRUNmode_80MHz: added () in while expression
 */

#include "S32K142.h"           /* include peripheral declarations S32K142 */
#include "clocks_and_modes.h"

void SOSC_init_8MHz(void)
{
  SCG->SOSCDIV=0x00000101;  		 	/* SOSCDIV1 & SOSCDIV2 =1: divide by 1 */
  SCG->SOSCCFG=0x00000024;  	 	 	/* Range=2: Medium freq (SOSC betw 1MHz-8MHz)*/
                            	 	 	/* HGO=0:   Config xtal osc for low power */
                            	 	 	/* EREFS=1: Input is external XTAL */
  while(SCG->SOSCCSR & SCG_SOSCCSR_LK_MASK); /* Ensure SOSCCSR unlocked */
  SCG->SOSCCSR=0x00000001;  	 	 	/* LK=0:          SOSCCSR can be written */
                            	 	 	/* SOSCCMRE=0:    OSC CLK monitor IRQ if enabled */
                            	 	 	/* SOSCCM=0:      OSC CLK monitor disabled */
                            	 	 	/* SOSCERCLKEN=0: Sys OSC 3V ERCLK output clk disabled */
                            	 	 	/* SOSCLPEN=0:    Sys OSC disabled in VLP modes */
                            	 	 	/* SOSCSTEN=0:    Sys OSC disabled in Stop modes */
                            	 	 	/* SOSCEN=1:      Enable oscillator */
  while(!(SCG->SOSCCSR & SCG_SOSCCSR_SOSCVLD_MASK)); /* Wait for sys OSC clk valid */
}

void SPLL_init_160MHz(void)
{
  while(SCG->SPLLCSR & SCG_SPLLCSR_LK_MASK); /* Ensure SPLLCSR unlocked */

  SCG->SPLLCSR = 0x00000000;  	 	 	/* SPLLEN=0: SPLL is disabled (default) */
  SCG->SPLLDIV = 0x00000302;  	 	 	/* SPLLDIV1 divide by 2; SPLLDIV2 divide by 4 */
  SCG->SPLLCFG = 0x00180000;  	 	 	/* PREDIV=0: Divide SOSC_CLK by 0+1=1 */
                              	 	 	/* MULT=11000b:  Multiply sys pll by 40 */
                              	 	 	/* SPLL_CLK = 8MHz / 1 * 40 / 2 = 160 MHz */

  while(SCG->SPLLCSR & SCG_SPLLCSR_LK_MASK); /* Ensure SPLLCSR unlocked */
  SCG->SPLLCSR = 0x00000001; 	 	 	/* LK=0:        SPLLCSR can be written */
                             	 	 	/* SPLLCMRE=0:  SPLL CLK monitor IRQ if enabled */
                             	 	 	/* SPLLCM=0:    SPLL CLK monitor disabled */
                             	 	 	/* SPLLSTEN=0:  SPLL disabled in Stop modes */
                             	 	 	/* SPLLEN=1:    Enable SPLL */
  while(!(SCG->SPLLCSR & SCG_SPLLCSR_SPLLVLD_MASK)); /* Wait for SPLL valid */
}

void NormalRUNmode_80MHz (void)  		/* Change to normal RUN mode with 8MHz SOSC, 80 MHz PLL*/
{
  SCG->RCCR=SCG_RCCR_SCS(6)      		/* PLL as clock source*/
    |SCG_RCCR_DIVCORE(0b01)      		/* DIVCORE=1, div. by 2: Core clock = 160/2 MHz = 80 MHz*/
    |SCG_RCCR_DIVBUS(0b01)       		/* DIVBUS=1, div. by 2: bus clock = 40 MHz*/
    |SCG_RCCR_DIVSLOW(0b10);     		/* DIVSLOW=2, div. by 2: SCG slow, flash clock= 26 2/3 MHz*/
  while (((SCG->CSR & SCG_CSR_SCS_MASK) >> SCG_CSR_SCS_SHIFT ) != 6) {}
                                 	 	/* Wait for sys clk src = SPLL */
}

void FIRC_init(void)
{
	  SCG->FIRCDIV=0x00000401;   		/* FIRCDIV2 =4 : divide by 08 48M/=6MHz used for ADC
	  	  	  	  	  	  	  	 	 	   FIRCDIV1 =1 : divide by 1 48M/1=48MHz used for PWM0 */

	  SCG->FIRCCFG =0x00000000;  		/* FIRC is trimmed to 48MHz */
	  while(SCG->FIRCCSR & SCG_FIRCCSR_LK_MASK);
	  SCG->FIRCCSR=0x00000001;  		/* LK=0:          	FIRCCSR can be written */
	                            		/* FIRCMRE=0:    	FIRC CLK monitor IRQ if enabled */
	                            		/* FIRCCM=0:      	FIRC CLK monitor disabled */
	                            		/* FIRCERCLKEN=0: 	FIRC 3V ERCLK output clk disabled */
	                            		/* FIRCLPEN=0:    	FIRC disabled in VLP modes */
	                            		/* FIRCSTEN=0:    	FIRC disabled in Stop modes */
	                            		/* FIRCEN=1:      	Enable FIRC */
	  while(!(SCG->FIRCCSR & SCG_FIRCCSR_FIRCVLD_MASK)); /* Wait for FIRC clk valid */
}

void FIRC_RUN_48MHz(void)
{
	  /* Change to FRC RUN mode with 48 MHz FRC CLOCK*/
	  SCG->RCCR=SCG_RCCR_SCS(3)      	/* PLL as clock source*/
	    |SCG_RCCR_DIVCORE(0b0)      	/* DIVCORE=0, div. by 1: Core clock = 48/1 MHz = 48 MHz*/
	    |SCG_RCCR_DIVBUS(0b0)       	/* DIVBUS=0, div. by 1: bus clock = 48 MHz*/
	    |SCG_RCCR_DIVSLOW(0b1);     	/* DIVSLOW=1, div. by 2: SCG slow, flash clock= 24 2/3 MHz*/
	  while (((SCG->CSR & SCG_CSR_SCS_MASK) >> SCG_CSR_SCS_SHIFT ) != 3) {}
	                                 	/* Wait for sys clk src = FRC */
}



/*******************************************************************************
Function Name : SOSC_Init
Engineer      : Petr Stancik
Date          : Oct-2-2016
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Initialize XOSC (8MHz), SPLL (160MHz) and RUN mode
Issues        : NONE
*******************************************************************************/
void SCG_Init(void)
{
  // *** System OSC ***
  SCG->SOSCDIV=0x00000101;  /* SOSCDIV1 & SOSCDIV2 =1: divide by 1 */
  SCG->SOSCCFG=0x00000034;  /* Range=3: high freq (SOSC betw 8MHz-32MHz)*/
                            /* HGO=0:   Config xtal osc for low power */
                            /* EREFS=1: Input is external XTAL */
  while(SCG->SOSCCSR & SCG_SOSCCSR_LK_MASK); /* Ensure SOSCCSR unlocked */
  SCG->SOSCCSR=0x00000001;  /* LK=0:          SOSCCSR can be written */
                            /* SOSCCMRE=0:    OSC CLK monitor IRQ if enabled */
                            /* SOSCCM=0:      OSC CLK monitor disabled */
                            /* SOSCERCLKEN=0: Sys OSC 3V ERCLK output clk disabled */
                            /* SOSCLPEN=0:    Sys OSC disabled in VLP modes */
                            /* SOSCSTEN=0:    Sys OSC disabled in Stop modes */
                            /* SOSCEN=1:      Enable oscillator */
  while(!(SCG->SOSCCSR & SCG_SOSCCSR_SOSCVLD_MASK)); /* Wait for sys OSC clk valid */

  // *** System PLL ***
  SCG->SPLLDIV = 0x00000202;  /* SOSCPLL1 & SPLLDIV2 =2: divide by 2 */

  SCG->SPLLCFG = 0x00180000;  /* PREDIV=0: Divide SOSC_CLK by 0+1=1 */
                              /* MULT=24:  Multiply sys pll by 24+16=40 */
                              /* SPLL_CLK = 8MHz / 1 * 40 / 2 = 160 MHz */

  while(SCG->SPLLCSR & SCG_SPLLCSR_LK_MASK); /* Ensure SPLLCSR unlocked */
  SCG->SPLLCSR = 0x00000001; /* LK=0:        SPLLCSR can be written */
                             /* SPLLCMRE=0:  SPLL CLK monitor IRQ if enabled */
                             /* SPLLCM=0:    SPLL CLK monitor disabled */
                             /* SPLLSTEN=0:  SPLL disabled in Stop modes */
                             /* SPLLEN=1:    Enable SPLL */
  while(!(SCG->SPLLCSR & SCG_SPLLCSR_SPLLVLD_MASK)); /* Wait for SPLL valid */

  // *** MODE CONFIG ***
  SCG->RCCR=SCG_RCCR_SCS(6)  /* PLL as clock source*/
	    |SCG_RCCR_DIVCORE(0b01)  /* DIVCORE= 2, Core/Sys clock = 80 MHz*/
	    |SCG_RCCR_DIVBUS(0b11)   /* DIVBUS = 4, bus clock = 40 MHz*/
	    |SCG_RCCR_DIVSLOW(0b111); /* DIVSLOW = 8, SCG slow, flash clock= 20 MHz*/

  SMC->PMCTRL  = SMC_PMCTRL_RUNM(0b00);	//enter RUN

}
